#import "MyController.h"
#import "NSImage-IconExtensions.h"

#define ICON_MAX	162

static OSType	iconTypes[] =
{
	// Alert Icons
	kAlertNoteIcon,
	kAlertCautionIcon,
	kAlertStopIcon,
	
	// Filesharing Privilege Icons
	kSharingPrivsNotApplicableIcon,
	kSharingPrivsReadOnlyIcon,
	kSharingPrivsReadWriteIcon,
	kSharingPrivsUnknownIcon,
	kSharingPrivsWritableIcon,
	
	// Folder Icons
	kGenericFolderIcon,
	kDropFolderIcon,
	kMountedFolderIcon,
	kOpenFolderIcon,
	kOwnedFolderIcon,
	kPrivateFolderIcon,
	
	// Internet Icons
	kInternetLocationHTTPIcon,
	kInternetLocationFTPIcon,
	kInternetLocationAppleShareIcon,
	kInternetLocationAppleTalkZoneIcon,
	kInternetLocationFileIcon,
	kInternetLocationMailIcon,
	kInternetLocationNewsIcon,
	kInternetLocationNSLNeighborhoodIcon,
	kInternetLocationGenericIcon,
	
	// Toolbar Icons
	kToolbarCustomizeIcon,
	kToolbarDeleteIcon,
	kToolbarFavoritesIcon,
	kToolbarHomeIcon,
	
	// Miscellaneous Icons
	kAppleLogoIcon,
	kAppleMenuIcon,
	kBackwardArrowIcon,
	kFavoriteItemsIcon,
	kForwardArrowIcon,
	kGridIcon,
	kHelpIcon,
	kKeepArrangedIcon,
	kLockedIcon,
	kNoFilesIcon,
	kNoFolderIcon,
	kNoWriteIcon,
	kProtectedApplicationFolderIcon,
	kProtectedSystemFolderIcon,
	kRecentItemsIcon,
	kShortcutIcon,
	kSortAscendingIcon,
	kSortDescendingIcon,
	kUnlockedIcon,
	kConnectToIcon,
	kGenericWindowIcon,
	kQuestionMarkIcon,
	kDeleteAliasIcon,
	kEjectMediaIcon,
	kBurningIcon,
	kRightContainerArrowIcon,
	
	// Networking Icons
	kAppleTalkIcon,
	kAppleTalkZoneIcon,
	kAFPServerIcon,
	kFTPServerIcon,
	kHTTPServerIcon,
	kGenericNetworkIcon,
	kIPFileServerIcon,
	
	// Special Folder Icons
	kAppearanceFolderIcon,
	kAppleExtrasFolderIcon,
	kAppleMenuFolderIcon,
	kApplicationsFolderIcon,
	kApplicationSupportFolderIcon,
	kAssistantsFolderIcon,
	kColorSyncFolderIcon,
	kContextualMenuItemsFolderIcon,
	kControlPanelDisabledFolderIcon,
	kControlPanelFolderIcon,
	kControlStripModulesFolderIcon,
	kDocumentsFolderIcon,
	kExtensionsDisabledFolderIcon,
	kExtensionsFolderIcon,
	kFavoritesFolderIcon,
	kFontsFolderIcon,
	kHelpFolderIcon,
	kInternetFolderIcon,
	kInternetPlugInFolderIcon,
	kInternetSearchSitesFolderIcon,
	kLocalesFolderIcon,
	kMacOSReadMeFolderIcon,
	kPublicFolderIcon,
	kPreferencesFolderIcon,
	kPrinterDescriptionFolderIcon,
	kPrinterDriverFolderIcon,
	kPrintMonitorFolderIcon,
	kRecentApplicationsFolderIcon,
	kRecentDocumentsFolderIcon,
	kRecentServersFolderIcon,
	kScriptingAdditionsFolderIcon,
	kSharedLibrariesFolderIcon,
	kScriptsFolderIcon,
	kShutdownItemsDisabledFolderIcon,
	kShutdownItemsFolderIcon,
	kSpeakableItemsFolder,
	kStartupItemsDisabledFolderIcon,
	kStartupItemsFolderIcon,
	kSystemExtensionDisabledFolderIcon,
	kSystemFolderIcon,
	kTextEncodingsFolderIcon,
	kUsersFolderIcon,
	kUtilitiesFolderIcon,
	kVoicesFolderIcon,
	
	// Standard Finder Icons
	kClipboardIcon,
	kClippingUnknownTypeIcon,
	kClippingPictureTypeIcon,
	kClippingTextTypeIcon,
	kClippingSoundTypeIcon,
	kDesktopIcon,
	kFinderIcon,
	kFontSuitcaseIcon,
	kFullTrashIcon,
	kGenericApplicationIcon,
	kGenericCDROMIcon,
	kGenericControlPanelIcon,
	kGenericControlStripModuleIcon,
	kGenericComponentIcon,
	kGenericDeskAccessoryIcon,
	kGenericDocumentIcon,
	kGenericEditionFileIcon,
	kGenericExtensionIcon,
	kGenericFileServerIcon,
	kGenericFontIcon,
	kGenericFontScalerIcon,
	kGenericFloppyIcon,
	kGenericHardDiskIcon,
	kGenericIDiskIcon,
	kGenericRemovableMediaIcon,
	kGenericMoverObjectIcon,
	kGenericPCCardIcon,
	kGenericPreferencesIcon,
	kGenericQueryDocumentIcon,
	kGenericRAMDiskIcon,
	kGenericSharedLibaryIcon,
	kGenericStationeryIcon,
	kGenericSuitcaseIcon,
	kGenericURLIcon,
	kGenericWORMIcon,
	kInternationalResourcesIcon,
	kKeyboardLayoutIcon,
	kSoundFileIcon,
	kSystemSuitcaseIcon,
	kTrashIcon,
	kTrueTypeFontIcon,
	kTrueTypeFlatFontIcon,
	kTrueTypeMultiFlatFontIcon,
	kUserIDiskIcon,
	kUnknownFSObjectIcon,
	kInternationResourcesIcon,
	
	// Standard Icon Badges
	kAppleScriptBadgeIcon,
	kLockedBadgeIcon,
	kMountedBadgeIcon,
	kSharedBadgeIcon,
	kAliasBadgeIcon,
	kAlertCautionBadgeIcon,
	
	// Users and Groups Icons
	kUserFolderIcon,
	kWorkgroupFolderIcon,
	kGuestUserIcon,
	kUserIcon,
	kOwnerIcon,
	kGroupIcon
};

static NSString	*iconTypeNames[] =
{
	// Alert Icons
	@"kAlertNoteIcon",
	@"kAlertCautionIcon",
	@"kAlertStopIcon",
	
	// Filesharing Privilege Icons
	@"kSharingPrivsNotApplicableIcon",
	@"kSharingPrivsReadOnlyIcon",
	@"kSharingPrivsReadWriteIcon",
	@"kSharingPrivsUnknownIcon",
	@"kSharingPrivsWritableIcon",
	
	// Folder Icons
	@"kGenericFolderIcon",
	@"kDropFolderIcon",
	@"kMountedFolderIcon",
	@"kOpenFolderIcon",
	@"kOwnedFolderIcon",
	@"kPrivateFolderIcon",
	
	// Internet Icons
	@"kInternetLocationHTTPIcon",
	@"kInternetLocationFTPIcon",
	@"kInternetLocationAppleShareIcon",
	@"kInternetLocationAppleTalkZoneIcon",
	@"kInternetLocationFileIcon",
	@"kInternetLocationMailIcon",
	@"kInternetLocationNewsIcon",
	@"kInternetLocationNSLNeighborhoodIcon",
	@"kInternetLocationGenericIcon",
	
	// Toolbar Icons
	@"kToolbarCustomizeIcon",
	@"kToolbarDeleteIcon",
	@"kToolbarFavoritesIcon",
	@"kToolbarHomeIcon",
	
	// Miscellaneous Icons
	@"kAppleLogoIcon",
	@"kAppleMenuIcon",
	@"kBackwardArrowIcon",
	@"kFavoriteItemsIcon",
	@"kForwardArrowIcon",
	@"kGridIcon",
	@"kHelpIcon",
	@"kKeepArrangedIcon",
	@"kLockedIcon",
	@"kNoFilesIcon",
	@"kNoFolderIcon",
	@"kNoWriteIcon",
	@"kProtectedApplicationFolderIcon",
	@"kProtectedSystemFolderIcon",
	@"kRecentItemsIcon",
	@"kShortcutIcon",
	@"kSortAscendingIcon",
	@"kSortDescendingIcon",
	@"kUnlockedIcon",
	@"kConnectToIcon",
	@"kGenericWindowIcon",
	@"kQuestionMarkIcon",
	@"kDeleteAliasIcon",
	@"kEjectMediaIcon",
	@"kBurningIcon",
	@"kRightContainerArrowIcon",
	
	// Networking Icons
	@"kAppleTalkIcon",
	@"kAppleTalkZoneIcon",
	@"kAFPServerIcon",
	@"kFTPServerIcon",
	@"kHTTPServerIcon",
	@"kGenericNetworkIcon",
	@"kIPFileServerIcon",
	
	// Special Folder Icons
	@"kAppearanceFolderIcon",
	@"kAppleExtrasFolderIcon",
	@"kAppleMenuFolderIcon",
	@"kApplicationsFolderIcon",
	@"kApplicationSupportFolderIcon",
	@"kAssistantsFolderIcon",
	@"kColorSyncFolderIcon",
	@"kContextualMenuItemsFolderIcon",
	@"kControlPanelDisabledFolderIcon",
	@"kControlPanelFolderIcon",
	@"kControlStripModulesFolderIcon",
	@"kDocumentsFolderIcon",
	@"kExtensionsDisabledFolderIcon",
	@"kExtensionsFolderIcon",
	@"kFavoritesFolderIcon",
	@"kFontsFolderIcon",
	@"kHelpFolderIcon",
	@"kInternetFolderIcon",
	@"kInternetPlugInFolderIcon",
	@"kInternetSearchSitesFolderIcon",
	@"kLocalesFolderIcon",
	@"kMacOSReadMeFolderIcon",
	@"kPublicFolderIcon",
	@"kPreferencesFolderIcon",
	@"kPrinterDescriptionFolderIcon",
	@"kPrinterDriverFolderIcon",
	@"kPrintMonitorFolderIcon",
	@"kRecentApplicationsFolderIcon",
	@"kRecentDocumentsFolderIcon",
	@"kRecentServersFolderIcon",
	@"kScriptingAdditionsFolderIcon",
	@"kSharedLibrariesFolderIcon",
	@"kScriptsFolderIcon",
	@"kShutdownItemsDisabledFolderIcon",
	@"kShutdownItemsFolderIcon",
	@"kSpeakableItemsFolder",
	@"kStartupItemsDisabledFolderIcon",
	@"kStartupItemsFolderIcon",
	@"kSystemExtensionDisabledFolderIcon",
	@"kSystemFolderIcon",
	@"kTextEncodingsFolderIcon",
	@"kUsersFolderIcon",
	@"kUtilitiesFolderIcon",
	@"kVoicesFolderIcon",
	
	// Standard Finder Icons
	@"kClipboardIcon",
	@"kClippingUnknownTypeIcon",
	@"kClippingPictureTypeIcon",
	@"kClippingTextTypeIcon",
	@"kClippingSoundTypeIcon",
	@"kDesktopIcon",
	@"kFinderIcon",
	@"kFontSuitcaseIcon",
	@"kFullTrashIcon",
	@"kGenericApplicationIcon",
	@"kGenericCDROMIcon",
	@"kGenericControlPanelIcon",
	@"kGenericControlStripModuleIcon",
	@"kGenericComponentIcon",
	@"kGenericDeskAccessoryIcon",
	@"kGenericDocumentIcon",
	@"kGenericEditionFileIcon",
	@"kGenericExtensionIcon",
	@"kGenericFileServerIcon",
	@"kGenericFontIcon",
	@"kGenericFontScalerIcon",
	@"kGenericFloppyIcon",
	@"kGenericHardDiskIcon",
	@"kGenericIDiskIcon",
	@"kGenericRemovableMediaIcon",
	@"kGenericMoverObjectIcon",
	@"kGenericPCCardIcon",
	@"kGenericPreferencesIcon",
	@"kGenericQueryDocumentIcon",
	@"kGenericRAMDiskIcon",
	@"kGenericSharedLibaryIcon",
	@"kGenericStationeryIcon",
	@"kGenericSuitcaseIcon",
	@"kGenericURLIcon",
	@"kGenericWORMIcon",
	@"kInternationalResourcesIcon",
	@"kKeyboardLayoutIcon",
	@"kSoundFileIcon",
	@"kSystemSuitcaseIcon",
	@"kTrashIcon",
	@"kTrueTypeFontIcon",
	@"kTrueTypeFlatFontIcon",
	@"kTrueTypeMultiFlatFontIcon",
	@"kUserIDiskIcon",
	@"kUnknownFSObjectIcon",
	@"kInternationResourcesIcon",
	
	// Standard Icon Badges
	@"kAppleScriptBadgeIcon",
	@"kLockedBadgeIcon",
	@"kMountedBadgeIcon",
	@"kSharedBadgeIcon",
	@"kAliasBadgeIcon",
	@"kAlertCautionBadgeIcon",
	
	// Users and Groups Icons
	@"kUserFolderIcon",
	@"kWorkgroupFolderIcon",
	@"kGuestUserIcon",
	@"kUserIcon",
	@"kOwnerIcon",
	@"kGroupIcon"	
};


@implementation MyController

- (id) init
{
	self = [super init];
	if (self != nil)
	{
		int	i;
		
		_iconNames = [[NSMutableArray alloc] init];
		
		for (i = 0; i < ICON_MAX; i++)
		{
			[_iconNames addObject:iconTypeNames[i]];
		}
		_searchList = nil;

	}
	return self;
}

- (NSArray *)iconNames
{
	return _iconNames;
}

- (void)setImages:(OSType)iconType
{
	NSImage	*image;
	
	image = [[[NSImage alloc] initWithSystemIconType:iconType] autorelease];
	[image setScalesWhenResized:NO];
	[image setSize:NSMakeSize(128.0,  128.0)];
	[_carbonImageView128 setImage:image];
	
	image = [[[NSImage alloc] initWithSystemIconType:iconType] autorelease];
	[image setScalesWhenResized:NO];	
	[image setSize:NSMakeSize(64.0,  64.0)];
	[_carbonImageView64 setImage:image];

	image = [[[NSImage alloc] initWithSystemIconType:iconType] autorelease];
	[image setScalesWhenResized:NO];	
	[image setSize:NSMakeSize(32.0, 32.0)];
	[_carbonImageView32 setImage:image];

	image = [[[NSImage alloc] initWithSystemIconType:iconType] autorelease];
	[image setScalesWhenResized:NO];	
	[image setSize:NSMakeSize(16.0,  16.0)];
	[_carbonImageView16 setImage:image];
	
	image = [[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(iconType)];	
	[image setScalesWhenResized:NO];
	[image setSize:NSMakeSize(128.0,  128.0)];
	[_cocoaImageView128 setImage:image];
	
	image = [[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(iconType)];
	[image setScalesWhenResized:NO];	
	[image setSize:NSMakeSize(64.0,  64.0)];
	[_cocoaImageView64 setImage:image];
	
	image = [[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(iconType)];
	[image setScalesWhenResized:NO];	
	[image setSize:NSMakeSize(32.0, 32.0)];
	[_cocoaImageView32 setImage:image];
	
	image = [[NSWorkspace sharedWorkspace] iconForFileType:NSFileTypeForHFSTypeCode(iconType)];
	[image setScalesWhenResized:NO];	
	[image setSize:NSMakeSize(16.0,  16.0)];
	[_cocoaImageView16 setImage:image];
}

- (IBAction)loadImage:(id)sender
{
	int		selectedRow;
	
	if (_searchList == nil)
	{
		[self setImages:iconTypes[[_tableView selectedRow]]];
	}
	else if ((selectedRow = [_tableView selectedRow]) != -1)
	{
		int	index;
		NSString	*name;
	
		name = [_searchList objectAtIndex:selectedRow];
		index = [_iconNames indexOfObject:name];
	
		[self setImages:iconTypes[index]];
	}
}

- (IBAction)search:(id)sender
{
	NSString	*term;
	
	term = [sender stringValue];
	if ([term isEqual:@""])
	{
		[_searchList release];
		_searchList = nil;
	}
	else
	{
		int			i, count;
		NSRange		range;

		[_searchList release];
		_searchList = [[NSMutableArray alloc] init];
		count = [_iconNames count];
		for (i = 0; i < count; i++)
		{
			range = [[_iconNames objectAtIndex:i] rangeOfString:term options:NSCaseInsensitiveSearch];
			if (range.location != NSNotFound)
			{
				[_searchList addObject:[_iconNames objectAtIndex:i]];
			}
		}
	}
	
	[_tableView reloadData];
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	if (_searchList == nil)
	{
		return ICON_MAX;
	}
	return [_searchList count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	if (_searchList == nil)
	{
		return iconTypeNames[rowIndex];
	}
	return [_searchList objectAtIndex:rowIndex];
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	[self loadImage:self];
}


@end
